@@ -6,12 +6,9 @@ from vedo.colors import printc, getColor
 from vedo.assembly import Assembly
 from vedo.mesh import Mesh, merge
 from vedo.pointcloud import Points
-from vedo.utils import mag, isSequence, make_ticks
-from vedo.utils import versor, ctf2lut, vtkVersionIsAtLeast
-from vedo.utils import precision as nrprecision
+import vedo.utils as utils
 import vedo.shapes as shapes
 import vedo.settings as settings
-import vedo.docs as docs
 from vedo.volume import Volume
 from vedo.tetmesh import TetMesh
 import numpy as np
@@ -19,31 +16,32 @@ import vtk
 
 __doc__ = (
     """
-Additional objects like axes, legends etc..
+Create additional objects like axes, legends, lights, etc..
 """
-    + docs._defs
+    + vedo.docs._defs
 )
 
 __all__ = [
-        "addLight",
-        "addScalarBar",
-        "addScalarBar3D",
-        "addSlider2D",
-        "addSlider3D",
-        "addButton",
-        "addCutterTool",
-        "addIcon",
-        "addLegend",
-        "buildAxes",
-        "buildRulerAxes",
-        "Goniometer",
-        "Ruler",
+            "addLight",
+            "addScalarBar",
+            "addScalarBar3D",
+            "addSlider2D",
+            "addSlider3D",
+            "addButton",
+            "addCutterTool",
+            "addIcon",
+            "addLegend",
+            "buildAxes",
+            "buildRulerAxes",
+            "Goniometer",
+            "Ruler",
         ]
 
 
 #####################################################################
 def Ruler(
     p1, p2,
+    unitScale=1,
     label="",
     s=None,
     font="",
@@ -62,6 +60,7 @@ def Ruler(
     Build a 3D ruler to indicate the distance of two points p1 and p2.
 
     :param str label: alternative fixed label to be shown
+    :param float unitScale: factor to scale units (e.g. μm to mm)
     :param float s: size of the label
     :param str font: font name
     :param float italic: italicness of the font [0,1]
@@ -74,25 +73,27 @@ def Ruler(
 
     |goniometer| |goniometer.py|_
     """
-    ncolls = len(settings.collectable_actors)
+    if unitScale != 1.0 and units == "":
+        raise ValueError("When setting 'unitScale' to a value other than 1, a 'units' arguments must be specified.")
+    # ncolls = len(settings.collectable_actors)
 
     if isinstance(p1, Points): p1 = p1.GetPosition()
     if isinstance(p2, Points): p2 = p2.GetPosition()
     if len(p1)==2: p1=[p1[0],p1[1],0.0]
     if len(p2)==2: p2=[p2[0],p2[1],0.0]
     p1, p2 = np.array(p1), np.array(p2)
-    q1, q2 = [0, 0, 0], [mag(p2 - p1), 0, 0]
+    q1, q2 = [0, 0, 0], [utils.mag(p2 - p1), 0, 0]
     q1, q2 = np.array(q1), np.array(q2)
     v = q2 - q1
-    d = mag(v)
+    d = utils.mag(v) * unitScale
 
     if s is None:
-        s = d*0.02
+        s = d*0.02*(1/unitScale)
 
     if not label:
         label = str(d)
         if precision:
-            label = nrprecision(d, precision)
+            label = utils.precision(d, precision)
     if prefix:
         label = prefix+ '~' + label
     if units:
@@ -111,23 +112,24 @@ def Ruler(
     lc1 = shapes.Line(q1 - v / 50, pc1)
     lc2 = shapes.Line(q2 + v / 50, pc2)
 
-    zs = np.array([0, d / 50, 0])
+    zs = np.array([0, d / 50 * (1/unitScale), 0])
     ml1 = shapes.Line(-zs, zs).pos(q1)
     ml2 = shapes.Line(-zs, zs).pos(q2)
     ml1.RotateZ(tickAngle-90)
     ml2.RotateZ(tickAngle-90)
 
-    c1 = shapes.Circle(q1, r=d / 180, res=20)
-    c2 = shapes.Circle(q2, r=d / 180, res=20)
+    c1 = shapes.Circle(q1, r=d / 180 * (1/unitScale), res=20)
+    c2 = shapes.Circle(q2, r=d / 180 * (1/unitScale), res=20)
 
     acts = [lb, lc1, lc2, c1, c2, ml1, ml2]
     macts = merge(acts).pos(p1).c(c).alpha(alpha)
     macts.GetProperty().LightingOff()
     macts.GetProperty().SetLineWidth(lw)
+    macts.UseBoundsOff()
     macts.base = q1
     macts.top = q2
     macts.orientation(p2 - p1, rotation=axisRotation).bc('t').pickable(False)
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
     return macts
 
 
@@ -183,7 +185,7 @@ def Goniometer(
 
     |goniometer| |goniometer.py|_
     """
-    ncolls = len(settings.collectable_actors)
+    # ncolls = len(settings.collectable_actors)
 
     if isinstance(p1, Points): p1 = p1.GetPosition()
     if isinstance(p2, Points): p2 = p2.GetPosition()
@@ -197,14 +199,14 @@ def Goniometer(
     ln = shapes.Line([p1,p2,p3], lw=lw, c=c).alpha(alpha).lighting('off')
     acts.append(ln)
 
-    va = versor(p1-p2)
-    vb = versor(p3-p2)
-    r = min(mag(p3-p2), mag(p1-p2))*arcSize
+    va = utils.versor(p1-p2)
+    vb = utils.versor(p3-p2)
+    r = min(utils.mag(p3-p2), utils.mag(p1-p2))*arcSize
     ptsarc = []
     res = 120
     imed = int(res/2)
     for i in range(res+1):
-        vi = versor(vb*i/res + va*(res-i)/res)
+        vi = utils.versor(vb*i/res + va*(res-i)/res)
         if i==imed: vc = np.array(vi)
         ptsarc.append(p2+vi*r)
     arc = shapes.Line(ptsarc).lw(lw).c(c).alpha(alpha).lighting('off')
@@ -212,13 +214,10 @@ def Goniometer(
 
     angle = np.arccos(np.dot(va,vb))*180/np.pi
 
-    lb = shapes.Text(prefix+nrprecision(angle,precision)+'º', s=r/12*s,
+    lb = shapes.Text(prefix+utils.precision(angle,precision)+'º', s=r/12*s,
                      font=font, italic=italic, justify="center")
     cr = np.cross(va,vb)
     lb.pos(p2+vc*r/1.75).orientation(cr*np.sign(cr[2]), rotation=rotation)
-    # lb.base=np.array([0,0,0])
-    # lb.top=np.array([1,0,0])
-    # lb.pos(p2+vc*r/1.75).orientation(va, rotation=0)
     lb.c(c).alpha(alpha).bc('tomato').lighting('off')
     acts.append(lb)
 
@@ -229,7 +228,7 @@ def Goniometer(
         acts.append(msh)
 
     asse = Assembly(acts)
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
     return asse
 
 
@@ -268,6 +267,8 @@ class Button:
         elif font.lower() == "arial":
             self.textproperty.SetFontFamilyToArial()
         else:
+            if not font:
+                font = settings.defaultFont
             self.textproperty.SetFontFamily(vtk.VTK_FONT_FILE)
             self.textproperty.SetFontFile(settings.fonts_path + font +'.ttf')
         self.textproperty.SetFontSize(size)
@@ -403,7 +404,7 @@ def addScalarBar(obj,
             return None
 
     elif isinstance(obj, (Volume, TetMesh)):
-        lut = ctf2lut(obj)
+        lut = utils.ctf2lut(obj)
 
     else:
         return obj
@@ -416,8 +417,20 @@ def addScalarBar(obj,
     sb.SetUseOpacity(useAlpha)
     sb.SetDrawFrame(0)
     sb.SetDrawBackground(0)
+    if lut.GetUseBelowRangeColor():
+        sb.DrawBelowRangeSwatchOn()
+        sb.SetBelowRangeAnnotation('')
+    if lut.GetUseAboveRangeColor():
+        sb.DrawAboveRangeSwatchOn()
+        sb.SetAboveRangeAnnotation('')
+    if lut.GetNanColor() != (0.5, 0.0, 0.0, 1.0):
+        sb.DrawNanAnnotationOn()
+        sb.SetNanAnnotation('nan')
 
     if title:
+        if "\\" in repr(title):
+            for r in shapes._reps:
+                title = title.replace(r[0], r[1])
         titprop = sb.GetTitleTextProperty()
         titprop.BoldOn()
         titprop.ItalicOff()
@@ -431,14 +444,6 @@ def addScalarBar(obj,
         sb.SetVerticalTitleSeparation(titleYOffset)
         sb.SetTitleTextProperty(titprop)
 
-    #     lpr = sb.GetAnnotationTextProperty ()# GetLabelTextProperty()
-    #     lpr.BoldOff()
-    #     lpr.ItalicOff()
-    #     lpr.ShadowOff()
-    #     lpr.SetColor(c)
-    #     lpr.SetFontFamily(vtk.VTK_FONT_FILE)
-    #     lpr.SetFontFile(settings.fonts_path + settings.defaultFont +'.ttf')
-
     sb.UnconstrainedFontSizeOn()
     sb.DrawAnnotationsOn()
     sb.DrawTickLabelsOn()
@@ -462,13 +467,12 @@ def addScalarBar(obj,
         sb.SetNumberOfLabels(nlabels)
 
     sctxt = sb.GetLabelTextProperty()
-    sb.SetAnnotationTextProperty(sctxt)
+    sctxt.SetFontFamily(vtk.VTK_FONT_FILE)
+    sctxt.SetFontFile(settings.fonts_path + settings.defaultFont +'.ttf')
     sctxt.SetColor(c)
     sctxt.SetShadow(0)
-    sctxt.SetFontFamily(0)
-    sctxt.SetItalic(0)
-    sctxt.SetBold(0)
-    sctxt.SetFontSize(titleFontSize)
+    sctxt.SetFontSize(titleFontSize-2)
+    sb.SetAnnotationTextProperty(sctxt)
     sb.PickableOff()
     obj.scalarbar = sb
     return sb
@@ -493,6 +497,9 @@ def addScalarBar3D(
     c=None,
     useAlpha=True,
     drawBox=True,
+    aboveText=None,
+    belowText=None,
+    nanText='NaN',
 ):
     """
     Draw a 3D scalar bar.
@@ -518,6 +525,7 @@ def addScalarBar3D(
     .. hint:: |scalarbars| |scalarbars.py|_
     """
     plt = settings.plotter_instance
+    # ncolls = len(settings.collectable_actors)
     if plt and c is None:  # automatic black or white
         c = (0.9, 0.9, 0.9)
         if np.sum(getColor(plt.backgrcol)) > 1.5:
@@ -533,16 +541,18 @@ def addScalarBar3D(
 
     if isinstance(obj, Points):
         lut = obj.mapper().GetLookupTable()
-        if not lut:
-            print("Error in addScalarBar3D: mesh has no lookup table.", [obj])
-            return None
-        vmin, vmax = obj.mapper().GetScalarRange()
-
-    elif isinstance(obj, (Volume, TetMesh)):
-        lut = ctf2lut(obj)
+        if not lut or lut.GetTable().GetNumberOfTuples() == 0:
+            obj.cmap('jet_r') # create the most similar to the default
+            # todo: grab the auto created default LUT (but where is it?)
+            #       cells or points?
+            lut = obj.mapper().GetLookupTable()
         vmin, vmax = lut.GetRange()
 
-    elif isSequence(obj):
+    elif isinstance(obj, (Volume, TetMesh)):
+        lut = utils.ctf2lut(obj)
+        vmin, vmax = lut.GetRange()
+
+    elif utils.isSequence(obj):
         vmin, vmax = np.min(obj), np.max(obj)
 
     else:
@@ -551,29 +561,27 @@ def addScalarBar3D(
 
     # build the color scale part
     scale = shapes.Grid([-sx *labelOffset, 0, 0], c=c, alpha=1,
-                        sx=sx, sy=sy, resx=1, resy=256)
+                        sx=sx, sy=sy,
+                        resx=1, resy=lut.GetTable().GetNumberOfTuples())
     scale.lw(0).wireframe(False)
-    cscals = scale.cellCenters()[:, 1]
-    scale.cmap(lut, cscals, on='cells')
-    scale.lighting('off')
+    cscals = np.linspace(vmin, vmax, lut.GetTable().GetNumberOfTuples())
+    scale.cmap(lut, cscals, on='cells').lighting('off')
     xbns = scale.xbounds()
 
     if pos is None:
         d=sx/2
         if title:
             d = np.sqrt((bns[1]-bns[0])**2+sy*sy)/20
-        pos=(bns[1]-xbns[0]+d,
-             (bns[2]+bns[3])/2,
-             bns[4])
+        pos = (bns[1]-xbns[0]+d, (bns[2]+bns[3])/2, bns[4])
 
     tacts = []
-    ticks_pos, ticks_txt = make_ticks(vmin, vmax, nlabels)
-    nlabels2 = len(ticks_pos)-1
+    ticks_pos, ticks_txt = utils.make_ticks(vmin, vmax, nlabels)
+
     for i, p in enumerate(ticks_pos):
         tx = ticks_txt[i]
         if i and tx:
             # build numeric text
-            y = -sy /2 + sy * i / nlabels2
+            y = (p - 0.5) *sy
             a = shapes.Text(tx, pos=[sx*labelOffset, y, 0], s=sy/60,
                             justify='center-left', c=c, italic=italic, font=labelFont)
             tacts.append(a)
@@ -590,6 +598,54 @@ def addScalarBar3D(
         t.pos(sx*titleXOffset,titleYOffset,0)
         tacts.append(t)
 
+    # build below scale
+    brect = None
+    if lut.GetUseBelowRangeColor():
+        r,g,b,alfa = lut.GetBelowRangeColor()
+        brect = shapes.Rectangle([-sx *labelOffset -sx/2, -sy/2-sx-sx*0.1, 0],
+                                 [-sx *labelOffset +sx/2, -sy/2   -sx*0.1, 0],
+                                 c=(r,g,b), alpha=alfa)
+        brect.lw(1).lc(c).lighting('off')
+        if belowText is None:
+           belowText = ' <'+str(vmin)
+        if belowText:
+            btx = shapes.Text(belowText, (0,0,0), s=sy/60,
+                              c=c, justify='center-left', italic=italic, font=labelFont)
+            btx.pos(sx*labelOffset, -sy/2-sx*0.66, 0)
+            tacts.append(btx)
+
+    # build above scale
+    arect = None
+    if lut.GetUseAboveRangeColor():
+        r,g,b,alfa = lut.GetAboveRangeColor()
+        arect = shapes.Rectangle([-sx *labelOffset -sx/2, sy/2   +sx*0.1, 0],
+                                 [-sx *labelOffset +sx/2, sy/2+sx+sx*0.1, 0],
+                                 c=(r,g,b), alpha=alfa)
+        arect.lw(1).lc(c).lighting('off')
+        if aboveText is None:
+            aboveText = ' >'+str(vmax)
+        if aboveText:
+            atx = shapes.Text(aboveText, (0,0,0), s=sy/60,
+                              c=c, justify='center-left', italic=italic, font=labelFont)
+            atx.pos(sx*labelOffset, sy/2+sx*0.66, 0)
+            tacts.append(atx)
+
+    # build NaN scale
+    nanrect = None
+    if lut.GetNanColor() != (0.5, 0.0, 0.0, 1.0):
+        nanshift = sx*0.1
+        if brect:
+            nanshift += sx
+        r,g,b,alfa = lut.GetNanColor()
+        nanrect = shapes.Rectangle([-sx *labelOffset -sx/2, -sy/2-sx-sx*0.1-nanshift, 0],
+                                   [-sx *labelOffset +sx/2, -sy/2   -sx*0.1-nanshift, 0],
+                                   c=(r,g,b), alpha=alfa)
+        nanrect.lw(1).lc(c).lighting('off')
+        nantx = shapes.Text(nanText, (0,0,0), s=sy/60,
+                            c=c, justify='center-left', italic=italic, font=labelFont)
+        nantx.pos(sx*labelOffset, -sy/2-sx*0.66-nanshift, 0)
+        tacts.append(nantx)
+
     if drawBox:
         tacts.append(scale.box().lw(0.1))
 
@@ -599,17 +655,18 @@ def addScalarBar3D(
     mtacts.PickableOff()
     scale.PickableOff()
 
-    sact = Assembly(scale, tacts)
+    sact = Assembly([scale, arect, brect, nanrect] + tacts)
     sact.SetPosition(pos)
     sact.PickableOff()
     sact.UseBoundsOff()
     sact.name = 'ScalarBar3D'
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
     return sact
 
 
 #####################################################################
 def addSlider2D(sliderfunc, xmin, xmax, value=None, pos=4,
-                title='', font='arial', titleSize=1, c=None, showValue=True):
+                title='', font='', titleSize=1, c=None, showValue=True):
     """Add a slider widget which can call an external custom function.
 
     :param sliderfunc: external function to be called by the widget
@@ -619,7 +676,7 @@ def addSlider2D(sliderfunc, xmin, xmax, value=None, pos=4,
     :param list pos: position corner number: horizontal [1-5] or vertical [11-15]
           it can also be specified by corners coordinates [(x1,y1), (x2,y2)]
     :param str title: title text
-    :param str font: title font [arial, courier]
+    :param str font: title font
     :param float titleSize: title text scale [1.0]
     :param bool showValue:  if true current value is shown
 
@@ -646,7 +703,7 @@ def addSlider2D(sliderfunc, xmin, xmax, value=None, pos=4,
     sliderRep.SetTubeWidth(0.0075)
     sliderRep.GetPoint1Coordinate().SetCoordinateSystemToNormalizedDisplay()
     sliderRep.GetPoint2Coordinate().SetCoordinateSystemToNormalizedDisplay()
-    if isSequence(pos):
+    if utils.isSequence(pos):
         sliderRep.GetPoint1Coordinate().SetValue(pos[0][0], pos[0][1])
         sliderRep.GetPoint2Coordinate().SetValue(pos[1][0], pos[1][1])
     elif pos == 1:  # top-left horizontal
@@ -705,16 +762,22 @@ def addSlider2D(sliderfunc, xmin, xmax, value=None, pos=4,
     sliderRep.GetTitleProperty().SetColor(c)
     sliderRep.GetTitleProperty().SetOpacity(1)
     sliderRep.GetTitleProperty().SetBold(0)
-    if font == 'courier':
+    if font == 'Courier':
         sliderRep.GetTitleProperty().SetFontFamilyToCourier()
-    elif font.lower() == "times":
+    elif font == "Times":
         sliderRep.GetTitleProperty().SetFontFamilyToTimes()
-    else:
+    elif font == "Arial":
         sliderRep.GetTitleProperty().SetFontFamilyToArial()
+    else:
+        font = settings.fonts_path + settings.defaultFont + '.ttf'
+        sliderRep.GetTitleProperty().SetFontFamily(vtk.VTK_FONT_FILE)
+        sliderRep.GetLabelProperty().SetFontFamily(vtk.VTK_FONT_FILE)
+        sliderRep.GetTitleProperty().SetFontFile(font)
+        sliderRep.GetLabelProperty().SetFontFile(font)
 
     if title:
         sliderRep.SetTitleText(title)
-        if not isSequence(pos):
+        if not utils.isSequence(pos):
             if isinstance(pos, int) and pos > 10:
                 sliderRep.GetTitleProperty().SetOrientation(90)
         else:
@@ -823,7 +886,7 @@ def addButton(
     bc=("dg", "dr"),
     pos=(20, 40),
     size=24,
-    font="arial",
+    font="Normografo",
     bold=False,
     italic=False,
     alpha=1,
@@ -864,12 +927,12 @@ def addCutterTool(mesh):
         return _addVolumeCutterTool(mesh)
     elif isinstance(mesh, vtk.vtkImageData):
         return _addVolumeCutterTool(Volume(mesh))
+        #########################################
 
     plt = settings.plotter_instance
-    if not plt.renderer:
-        save_int = plt.interactive
-        plt.show(interactive=0)
-        plt.interactive = save_int
+    if not plt:
+        printc("addCutterTool(): scene must be first rendered.", c='r')
+        raise RuntimeError()
 
     plt.clickedActor = mesh
     apd = mesh.polydata()
@@ -878,7 +941,7 @@ def addCutterTool(mesh):
     planes.SetBounds(apd.GetBounds())
 
     clipper = vtk.vtkClipPolyData()
-    clipper.GenerateClipScalarsOn()
+    clipper.GenerateClipScalarsOff()
     clipper.SetInputData(apd)
     clipper.SetClipFunction(planes)
     clipper.InsideOutOn()
@@ -891,13 +954,11 @@ def addCutterTool(mesh):
     act0.mapper().SetScalarRange(mesh.mapper().GetScalarRange())
 
     act1 = Mesh()
-    act1.mapper().SetInputConnection(clipper.GetClippedOutputPort()) # needs OutputPort??
+    act1.mapper().SetInputConnection(clipper.GetClippedOutputPort()) # needs OutputPort
     act1.alpha(0.04).color((0.5,0.5,0.5)).wireframe()
 
-    plt.renderer.RemoveActor(mesh)
-    plt.renderer.AddActor(act0)
-    plt.renderer.AddActor(act1)
-    plt.renderer.ResetCamera()
+    plt.remove(mesh)
+    plt.add([act0, act1])
 
     def selectPolygons(vobj, event):
         vobj.GetPlanes(planes)
@@ -907,12 +968,13 @@ def addCutterTool(mesh):
     boxWidget.GetSelectedOutlineProperty().SetColor(1, 0, 1)
     boxWidget.GetOutlineProperty().SetColor(0.2, 0.2, 0.2)
     boxWidget.GetOutlineProperty().SetOpacity(0.8)
-    boxWidget.SetPlaceFactor(1.05)
+    boxWidget.SetPlaceFactor(1.025)
     boxWidget.SetInteractor(plt.interactor)
     boxWidget.SetInputData(apd)
     boxWidget.PlaceWidget()
     boxWidget.AddObserver("InteractionEvent", selectPolygons)
     boxWidget.On()
+    plt.widgets.append(boxWidget)
 
     plt.cutterWidget = boxWidget
     plt.clickedActor = act0
@@ -923,12 +985,9 @@ def addCutterTool(mesh):
     printc("Mesh Cutter Tool:", c="m", invert=1)
     printc("  Move gray handles to cut away parts of the mesh", c="m")
     printc("  Press X to save file to: clipped.vtk", c="m")
-    plt.interactor.Start()
+    plt.interactor.Start() # allow extra interaction
 
     boxWidget.Off()
-    plt.widgets.append(boxWidget)
-
-    plt.interactor.Start()  # allow extra interaction
     return act0
 
 def _addVolumeCutterTool(vol):
@@ -990,7 +1049,7 @@ def addIcon(mesh, pos=3, size=0.08):
     widget = vtk.vtkOrientationMarkerWidget()
     widget.SetOrientationMarker(mesh)
     widget.SetInteractor(plt.interactor)
-    if isSequence(pos):
+    if utils.isSequence(pos):
         widget.SetViewport(pos[0] - size, pos[1] - size, pos[0] + size, pos[1] + size)
     else:
         if pos < 2:
@@ -1010,7 +1069,7 @@ def addIcon(mesh, pos=3, size=0.08):
 
 #####################################################################
 def computeVisibleBounds():
-    """Calculate max meshs bounds and sizes."""
+    """Calculate max meshes bounds and sizes."""
     bns = []
     for a in settings.plotter_instance.actors:
         if a and a.GetUseBounds():
@@ -1054,7 +1113,7 @@ def buildRulerAxes(
     xycross=True,
 ):
     """
-    Build a 3D ruler to indicate the distance of two points p1 and p2.
+    Build 3D rulers to indicate the sizes of the input scene or object.
 
     :param str xtitle: name of the axis or title
     :param str xlabel: alternative fixed label to be shown instead of the distance
@@ -1071,7 +1130,8 @@ def buildRulerAxes(
 
     |goniometer| |goniometer.py|_
     """
-    if isSequence(inputobj):
+    # ncolls = len(settings.collectable_actors)
+    if utils.isSequence(inputobj):
         x0,x1,y0,y1,z0,z1 = inputobj
     else:
         x0,x1,y0,y1,z0,z1 = inputobj.GetBounds()
@@ -1087,24 +1147,26 @@ def buildRulerAxes(
                     [x1,y0-dx,z0],
                     s=s, font=font, precision=precision,
                     labelRotation=labelRotation, axisRotation=axisRotation,
-                    lw=lw, italic=italic, prefix=xtitle, label=xlabel, units=units)
+                    lw=lw, italic=italic, prefix=xtitle, label=xlabel, units=units
+        )
         acts.append(rx)
     if ytitle is not None and (y1-y0)/d>0.1:
         ry = Ruler( [x1+dy,y0,z0],
                     [x1+dy,y1,z0],
                     s=s, font=font, precision=precision,
                     labelRotation=labelRotation, axisRotation=axisRotation,
-                    lw=lw, italic=italic, prefix=ytitle, label=ylabel, units=units)
+                    lw=lw, italic=italic, prefix=ytitle, label=ylabel, units=units
+        )
         acts.append(ry)
     if ztitle is not None and (z1-z0)/d>0.1:
         rz = Ruler( [x0-dy,y0+dz,z0],
                     [x0-dy,y0+dz,z1],
                     s=s, font=font, precision=precision,
                     labelRotation=labelRotation, axisRotation=axisRotation+90,
-                    lw=lw, italic=italic, prefix=ztitle, label=zlabel, units=units)
+                    lw=lw, italic=italic, prefix=ztitle, label=zlabel, units=units
+        )
         acts.append(rz)
 
-    ncolls = len(settings.collectable_actors)
     if xycross and rx and ry:
         lx = shapes.Line([x0,y0,z0],    [x0,y1+dx,z0])
         ly = shapes.Line([x0-dy,y1,z0], [x1,y1,z0])
@@ -1114,7 +1176,7 @@ def buildRulerAxes(
 
     macts = merge(acts).c(c).alpha(alpha).bc('t')
     macts.UseBoundsOff()
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
     return macts
 
 
@@ -1129,6 +1191,15 @@ def buildAxes(obj=None,
               axesLineWidth=1,
               gridLineWidth=1,
               reorientShortTitle=True,
+              htitle="",
+              hTitleSize=0.03,
+              hTitleFont=None,
+              hTitleItalic=True,
+              hTitleColor=None,
+              hTitleJustify='bottom-center',
+              hTitleHorizontalOffset=0,
+              hTitleVerticalOffset=0.01,
+              hTitleZOffset=0,
               titleDepth=0,
               titleFont="", # grab settings.defaultFont
               textScale=1.0,
@@ -1192,6 +1263,15 @@ def buildAxes(obj=None,
     - `titleFont`,              [''], font for axes titles
     - `labelFont`,              [''], font for numeric labels
     - `textScale`,             [1.0], global scaling factor for text elements (titles, labels)
+    - `htitle`,                 [''], header title
+    - `hTitleSize`,           [0.03], header title size
+    - `hTitleFont`,           [None], header font (defaults to `titleFont`)
+    - `hTitleItalic`,         [True], header font is italic
+    - `hTitleColor`,          [None], header title color (defaults to `xTitleColor`)
+    - `hTitleJustify`, ['bottom-center'], origin of the title justification
+    - `hTitleHorizontalOffset`,  [0], control vertical positioning of header title
+    - `hTitleVerticalOffset`, [0.01], control horizontal positioning of header title
+    - `hTitleZOffset`,           [0], control positioning of header title along z-axis
     - `xTitlePosition`,       [0.32], title fractional positions along axis
     - `xTitleOffset`,         [0.05], title fractional offset distance from axis line
     - `xTitleJustify`, ["top-right"], title justification
@@ -1228,7 +1308,7 @@ def buildAxes(obj=None,
 
     |customAxes| |customAxes.py|_
     """
-    ncolls = len(settings.collectable_actors)
+    # ncolls = len(settings.collectable_actors)
 
     if not titleFont:
         titleFont = settings.defaultFont
@@ -1249,6 +1329,7 @@ def buildAxes(obj=None,
 
     if useGlobal:
         vbb, ss, min_bns, max_bns = computeVisibleBounds()
+        vbb = list(vbb) # so that we can modify it
     else:
         if obj:
             vbb = list(obj.GetBounds())
@@ -1258,7 +1339,9 @@ def buildAxes(obj=None,
         else:
             vbb = np.zeros(6)
             ss = np.zeros(3)
-            if xrange is None or yrange is None or zrange is None:
+            if zrange is None:
+                zrange=(0,0)
+            if xrange is None or yrange is None:
                 printc("ERROR in buildAxes(): no mesh given, so must specify ranges.", c='r')
                 raise RuntimeError()
 
@@ -1313,7 +1396,7 @@ def buildAxes(obj=None,
     if not zLabelColor:  zLabelColor = zLineColor
 
     # vtk version<9 dont like depthpeeling
-    if settings.useDepthPeeling and not vtkVersionIsAtLeast(9):
+    if settings.useDepthPeeling and not utils.vtkVersionIsAtLeast(9):
         xyGrid = False
         yzGrid = False
         zxGrid = False
@@ -1339,11 +1422,11 @@ def buildAxes(obj=None,
     #printc('numberOfDivisions', numberOfDivisions, '\t r=', rx, ry, rz)
 
     if xtitle:
-        xticks_float, xticks_str = make_ticks(vbb[0], vbb[1], rx,
-                                              xValuesAndLabels, digits)
+        xticks_float, xticks_str = utils.make_ticks(vbb[0], vbb[1], rx,
+                                                    xValuesAndLabels, digits)
     if ytitle:
-        yticks_float, yticks_str = make_ticks(vbb[2], vbb[3], ry,
-                                              yValuesAndLabels, digits)
+        yticks_float, yticks_str = utils.make_ticks(vbb[2], vbb[3], ry,
+                                                    yValuesAndLabels, digits)
         if yLabelRotation is None: #automatic rotation of labels
             maxlentxt=0
             for i in range(1,len(yticks_str)-1):
@@ -1351,8 +1434,8 @@ def buildAxes(obj=None,
             if maxlentxt < 4:
                 yLabelRotation = 1 # rotate clockwise
     if ztitle:
-        zticks_float, zticks_str = make_ticks(vbb[4], vbb[5], rz,
-                                              zValuesAndLabels, digits)
+        zticks_float, zticks_str = utils.make_ticks(vbb[4], vbb[5], rz,
+                                                    zValuesAndLabels, digits)
         if zLabelRotation is None: #automatic rotation of labels
             maxlentxt=0
             for i in range(1,len(zticks_str)-1):
@@ -1756,12 +1839,29 @@ def buildAxes(obj=None,
         zt.UseBoundsOff()
         zt.name = "ztitle "+str(ztitle)
         titles.append(zt)
-    ###################################################
+
+    ################################################### header title
+    if htitle:
+        if hTitleFont is None:
+            hTitleFont = titleFont
+        if hTitleColor is None:
+            hTitleColor = xTitleColor
+        htit = shapes.Text(htitle, s=hTitleSize, font=hTitleFont,
+                           c=hTitleColor, justify=hTitleJustify, depth=titleDepth,
+                           italic=hTitleItalic,
+                           )
+        wpos = [0.5+hTitleHorizontalOffset, 1+hTitleVerticalOffset, hTitleZOffset]
+        htit.SetScale(x_aspect_ratios)
+        htit.pos(wpos)
+        # htit.UseBoundsOff()
+        htit.name = "htitle "+str(htitle)
+        titles.append(htit)
+
 
     acts = titles + lines + labels + grids + grids2 + highlights
     acts += framelines + majorticks + minorticks + cones
 
-    tol *= mag(ss)
+    tol *= utils.mag(ss)
     orig = np.array([min_bns[0], min_bns[2], min_bns[4]]) - tol
     for a in acts:
         a.PickableOff()
@@ -1772,7 +1872,7 @@ def buildAxes(obj=None,
     asse.SetScale(ss)
     asse.PickableOff()
     # throw away all extra created obj in collectable_actors
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
     return asse
 
 
@@ -1798,14 +1898,12 @@ def addGlobalAxes(axtype=None, c=None):
 
     Axis type-1 can be fully customized by passing a dictionary ``axes=dict()`` where:
 
-
         - `xtitle`,                ['x'], x-axis title text
         - `xrange`,               [None], x-axis range in format (xmin, ymin), default is automatic.
         - `numberOfDivisions`,    [None], approximate number of divisions on the longest axis
         - `axesLineWidth`,           [1], width of the axes lines
         - `gridLineWidth`,           [1], width of the grid lines
         - `reorientShortTitle`,   [True], titles shorter than 2 letter are placed horizontally
-        - `originMarkerSize`,     [0.01], draw a small cube on the axis where the origin is
         - `titleDepth`,              [0], extrusion fractional depth of title text
         - `xyGrid`,               [True], show a gridded wall on plane xy
         - `yzGrid`,               [True], show a gridded wall on plane yz
@@ -1818,25 +1916,33 @@ def addGlobalAxes(axtype=None, c=None):
         - `xyAlpha`,              [0.15], grid plane opacity
         - `xyFrameLine`,          [None], add a frame for the plane
         - `showTicks`,            [True], show major ticks
+        - `digits`,               [None], use this number of significant digits in scientific notation
+        - `titleFont`,              [''], font for axes titles
+        - `labelFont`,              [''], font for numeric labels
+        - `textScale`,             [1.0], global scaling factor for text elements (titles, labels)
         - `xTitlePosition`,       [0.32], title fractional positions along axis
         - `xTitleOffset`,         [0.05], title fractional offset distance from axis line
         - `xTitleJustify`, ["top-right"], title justification
         - `xTitleRotation`,          [0], add a rotation of the axis title
+        - `xTitleBox`,           [False], add a box around title text
         - `xLineColor`,      [automatic], color of the x-axis
         - `xTitleColor`,     [automatic], color of the axis title
         - `xTitleBackfaceColor`,  [None],  color of axis title on its backface
         - `xTitleSize`,          [0.025], size of the axis title
+        - 'xTitleItalic',            [0], a bool or float to make the font italic
         - `xHighlightZero`,       [True], draw a line highlighting zero position if in range
         - `xHighlightZeroColor`, [autom], color of the line highlighting the zero position
         - `xTickLength`,         [0.005], radius of the major ticks
         - `xTickThickness`,     [0.0025], thickness of the major ticks along their axis
-        - `xLabelColor`,      [automatic], color of major ticks
         - `xMinorTicks`,             [1], number of minor ticks between two major ticks
-        - `tipSize`,              [0.01], size of the arrow tip
-        - `xValuesAndLabels`       [], assign custom tick positions and labels [(pos1, label1), ...]
+        - `xValuesAndLabels`          [], assign custom tick positions and labels [(pos1, label1), ...]
+        - `xLabelColor`,     [automatic], color of numeric labels and ticks
         - `xLabelPrecision`,         [2], nr. of significative digits to be shown
         - `xLabelSize`,          [0.015], size of the numeric labels along axis
+        - 'xLabelRotation',          [0], rotate clockwise [1] or anticlockwise [-1] by 90 degrees
+        - 'xFlipText',           [False], flip axis title and numeric labels orientation
         - `xLabelOffset`,        [0.025], offset of numeric labels
+        - `tipSize`,              [0.01], size of the arrow tip
         - `limitRatio`,           [0.04], below this ratio don't plot small axis
 
         :Example:
@@ -2185,6 +2291,9 @@ def addGlobalAxes(axtype=None, c=None):
         ls.GetBottomAxis().GetLabelTextProperty().BoldOff()
         ls.GetBottomAxis().GetLabelTextProperty().ItalicOff()
         ls.GetBottomAxis().GetLabelTextProperty().ShadowOff()
+        pr = ls.GetBottomAxis().GetLabelTextProperty()
+        pr.SetFontFamily(vtk.VTK_FONT_FILE)
+        pr.SetFontFile(settings.fonts_path + settings.defaultFont +'.ttf')
         ls.PickableOff()
         plt.renderer.AddActor(ls)
         plt.axes_instances[r] = ls
@@ -2261,7 +2370,7 @@ def addRendererFrame(c=None, alpha=None, lw=None):
 def addLegend():
 
     plt = settings.plotter_instance
-    if not isSequence(plt._legend):
+    if not utils.isSequence(plt._legend):
         return
 
     # remove old legend if present on current renderer:
