@@ -1,8 +1,9 @@
 from __future__ import division, print_function
 import vtk
 import numpy as np
-import sys
+import sys, os, time
 import vedo.docs as docs
+import vedo.settings as settings
 
 __doc__ = (
     """
@@ -12,15 +13,16 @@ Colors definitions and printing methods.
 )
 
 __all__ = [
+    "ask",
     "printc",
     "getColor",
     "getColorName",
     "colorMap",
-    "makePalette",
-    "makeLUT",
-    "kelvin2rgb",
+    "buildPalette",
+    "buildLUT",
 ]
 
+_global_start_time = time.time()
 
 try:
     import matplotlib
@@ -394,8 +396,8 @@ def rgb2int(rgb_tuple):
     """Return the int number of a color from (r,g,b), with 0<r<1 etc."""
     rgb = (int(rgb_tuple[0] * 255), int(rgb_tuple[1] * 255), int(rgb_tuple[2] * 255))
     return 65536 * rgb[0] + 256 * rgb[1] + rgb[2]
-#    r,g,b = np.array(rgb_tuple, dtype=np.int)*255
-#    return (r << 16) + (g << 8) + b
+    #r,g,b = np.array(rgb_tuple, dtype=np.int)*255
+    #return (r << 16) + (g << 8) + b
 
 
 def colorMap(value, name="jet", vmin=None, vmax=None):
@@ -463,7 +465,7 @@ def colorMap(value, name="jet", vmin=None, vmax=None):
         return mp(value)[0:3]
 
 
-def makePalette(color1, color2, N, hsv=True):
+def buildPalette(color1, color2, N, hsv=True):
     """
     Generate N colors starting from `color1` to `color2`
     by linear interpolation HSV in or RGB spaces.
@@ -486,54 +488,61 @@ def makePalette(color1, color2, N, hsv=True):
         if hsv:
             c = np.array(hsv2rgb(c))
         cols.append(c)
-    return cols
+    return np.array(cols)
 
 
-def makeLUT(colorlist,
-            interpolate=False,
+def buildLUT(colorlist,
             vmin=None, vmax=None,
-            belowColor=None, aboveColor=None, nanColor=None
-            ):
+            belowColor=None, aboveColor=None, nanColor=None,
+            belowAlpha=1, aboveAlpha=1, nanAlpha=1,
+            interpolate=False,
+    ):
     """
-    Generate colors in a vtk lookup table.
+    Generate colors in a lookup table (LUT).
 
     :param list colorlist: a list in the form ``[(scalar1, [r,g,b]), (scalar2, 'blue'), ...]``.
-    :param bool interpolate: interpolate or not intermediate scalars
+
     :param float vmin: specify minimum value of scalar range
+
     :param float vmax: specify maximum value of scalar range
+
     :param belowColor: color for scalars below the minimum in range
+
+    :param belowAlpha: alpha for scalars below the minimum in range
+
     :param aboveColor: color for scalars above the maximum in range
 
-    :return: the lookup table object ``vtkLookupTable``. This can be fed into ``colorMap``.
+    :param aboveAlpha: alpha for scalars above the maximum in range
+
+    :param nanColor: color for invalid (nan) scalars
+
+    :param nanAlpha: alpha for invalid (nan) scalars
+
+    :param bool interpolate: interpolate or not intermediate scalars
+
+    :return: the lookup table object ``vtkLookupTable``. This can be fed into ``cmap``.
 
     .. hint:: Example: |mesh_lut.py|_
     """
-    lut = vtk.vtkLookupTable()
-    lut.SetNumberOfTableValues(256)
-    if nanColor is not None:
-        lut.SetNanColor(list(getColor(nanColor))+[1])
-
     ctf = vtk.vtkColorTransferFunction()
     ctf.SetColorSpaceToRGB()
     ctf.SetScaleToLinear()
-    if belowColor is not None:
-        ctf.SetBelowRangeColor(getColor(belowColor))
-        ctf.SetUseBelowRangeColor(True)
-        rgba = list(getColor(belowColor)) + [1]
-    if aboveColor is not None:
-        ctf.SetAboveRangeColor(getColor(aboveColor))
-        ctf.SetUseAboveRangeColor(True)
-        rgba = list(getColor(aboveColor)) + [1]
-
+    alpha_x, alpha_vals = [], []
     for sc in colorlist:
         if len(sc)==3:
-            scalar, col, _ = sc
+            scalar, col, alf = sc
         else:
+            alf = 1
             scalar, col = sc
         r, g, b = getColor(col)
         ctf.AddRGBPoint(scalar, r, g, b)
+        alpha_x.append(scalar)
+        alpha_vals.append(alf)
 
-    x0, x1 = ctf.GetRange()
+    lut = vtk.vtkLookupTable()
+    lut.SetNumberOfTableValues(256)
+
+    x0, x1 = ctf.GetRange() # range of the introduced values
     if vmin is not None:
         x0 = vmin
     if vmax is not None:
@@ -541,20 +550,30 @@ def makeLUT(colorlist,
     ctf.SetRange(x0, x1)
     lut.SetRange(x0, x1)
 
+    if belowColor is not None:
+        lut.SetBelowRangeColor(list(getColor(belowColor))+[belowAlpha])
+        lut.SetUseBelowRangeColor(True)
+    if aboveColor is not None:
+        lut.SetAboveRangeColor(list(getColor(aboveColor))+[aboveAlpha])
+        lut.SetUseAboveRangeColor(True)
+    if nanColor is not None:
+        lut.SetNanColor(list(getColor(nanColor))+[nanAlpha])
+
+    rgba = (1,1,1,1)
     for i in range(256):
         p = i/255
         x = (1-p) *x0 + p *x1
         if interpolate:
-            rgba = list(ctf.GetColor(x)) + [1]
+            alf = np.interp(x, alpha_x, alpha_vals)
+            rgba = list(ctf.GetColor(x)) + [alf]
         else:
-            rgba = [0.5,0.5,0.5,1]
             for c in colorlist:
                 if x <= c[0]:
                     if len(c)==3:
-                        al = c[2]
+                        alf = c[2]
                     else:
-                        al = 1
-                    rgba = list(getColor(c[1])) + [al]
+                        alf = 1
+                    rgba = list(getColor(c[1])) + [alf]
                     break
         lut.SetTableValue(i, rgba)
 
@@ -562,67 +581,6 @@ def makeLUT(colorlist,
     return lut
 
 
-def kelvin2rgb(temperature):
-    """
-    Converts from Kelvin temperature to an RGB color.
-
-    Algorithm credits: |tannerhelland|_
-    """
-    # range check
-    if temperature < 1000:
-        temperature = 1000
-    elif temperature > 40000:
-        temperature = 40000
-
-    tmp_internal = temperature / 100.0
-
-    # red
-    if tmp_internal <= 66:
-        red = 255
-    else:
-        tmp_red = 329.698727446 * np.power(tmp_internal - 60, -0.1332047592)
-        if tmp_red < 0:
-            red = 0
-        elif tmp_red > 255:
-            red = 255
-        else:
-            red = tmp_red
-
-    # green
-    if tmp_internal <= 66:
-        tmp_green = 99.4708025861 * np.log(tmp_internal) - 161.1195681661
-        if tmp_green < 0:
-            green = 0
-        elif tmp_green > 255:
-            green = 255
-        else:
-            green = tmp_green
-    else:
-        tmp_green = 288.1221695283 * np.power(tmp_internal - 60, -0.0755148492)
-        if tmp_green < 0:
-            green = 0
-        elif tmp_green > 255:
-            green = 255
-        else:
-            green = tmp_green
-
-    # blue
-    if tmp_internal >= 66:
-        blue = 255
-    elif tmp_internal <= 19:
-        blue = 0
-    else:
-        tmp_blue = 138.5177312231 * np.log(tmp_internal - 10) - 305.0447927307
-        if tmp_blue < 0:
-            blue = 0
-        elif tmp_blue > 255:
-            blue = 255
-        else:
-            blue = tmp_blue
-
-    return [red / 255, green / 255, blue / 255]
-
-
 # default sets of colors
 colors1 = [
     [1.0, 0.832, 0.000],  # gold
@@ -731,11 +689,17 @@ def printc(*strings, **keys):
     end = keys.pop("end", "\n")
     flush = keys.pop("flush", True)
 
-    if not _terminal_has_colors or sys.version_info[0] < 3:
+    if sys.version_info[0] < 3:
         print(strings)
-        sys.stdout.flush()
         return
 
+    if not settings.notebookBackend:
+        if not _terminal_has_colors:
+            print(end=end)
+            if flush:
+                sys.stdout.flush()
+            return
+
     c = keys.pop("c", None)
     bc = keys.pop("bc", None)
     bold = keys.pop("bold", True)
@@ -746,6 +710,7 @@ def printc(*strings, **keys):
     dim = keys.pop("dim", False)
     invert = keys.pop("invert", False)
     box = keys.pop("box", "")
+    dbg = keys.pop("dbg", False)
 
     if c is True:
         c = "green"
@@ -760,7 +725,7 @@ def printc(*strings, **keys):
     if bc is not None:
         bc = getColor(bc)
 
-    try:
+    try: # -------------------------------------------------------------
 
         txt = str()
         ns = len(strings) - 1
@@ -769,12 +734,17 @@ def printc(*strings, **keys):
         for i, s in enumerate(strings):
             if i == ns:
                 separator = ""
-            # txt += str(s) + separator
             if "\\" in repr(s):  # "in" for some reasons changes s
+                from vedo.shapes import _reps
                 for k in emoji.keys():
                     if k in str(s):
                         s = s.replace(k, emoji[k])
                         offset += 1
+                for k, rp in _reps: # check symbols in shapes._reps
+                    if k in str(s):
+                        s = s.replace(k, rp)
+                        offset += 1
+
             txt += str(s) + separator
 
         special, cseq = "", ""
@@ -821,11 +791,66 @@ def printc(*strings, **keys):
                 outtxt += box * (len(txt) + offset + 4) + "\x1b[0m" + end
 
             sys.stdout.write(outtxt)
-        else:
-            sys.stdout.write(special + cseq + txt + "\x1b[0m" + end)
 
-    except:
+        else:
+
+            out = special + cseq + txt + "\x1b[0m"
+
+            if dbg:
+                from inspect import currentframe, getframeinfo
+                from vedo.utils import isSequence, precision
+                cf = currentframe().f_back
+                cfi = getframeinfo(cf)
+                fname = os.path.basename(getframeinfo(cf).filename)
+                print("\x1b[7m\x1b[3m\x1b[37m"+fname+" line:\x1b[1m"+str(cfi.lineno)+"\x1b[0m", end='')
+                print('\x1b[3m\x1b[37m\x1b[2m', "\U00002501"*30, time.ctime(), "\x1b[0m")
+                if txt: print("    \x1b[37m\x1b[1mMessage : "+ out)
+                print("    \x1b[37m\x1b[1mFunction:\x1b[0m\x1b[37m "+ str(cfi.function))
+                print('    \x1b[1mLocals:\x1b[0m')
+                for loc in cf.f_locals.keys():
+                    obj = cf.f_locals[loc]
+                    var = repr(obj)
+                    if 'module ' in var: continue
+                    if 'function ' in var: continue
+                    if 'class ' in var: continue
+                    if '_' in loc: continue
+                    if hasattr(obj, 'name'):
+                        if not obj.name:
+                            oname = str(type(obj))
+                        else:
+                            oname = obj.name
+                        var = oname + ', at ' + precision(obj.GetPosition(),3)
+
+                    print('      \x1b[37m', loc,'=', var[:60].replace('\n',''), '\x1b[0m')
+                    if isSequence(obj) and len(obj)>4:
+                        print('           \x1b[37m\x1b[2m\x1b[3m len:', len(obj),
+                              ' min:', precision(min(obj), 4),
+                              ' max:', precision(max(obj), 4),
+                               # ' mean:', np.mean(np.array(obj)),
+                              '\x1b[0m')
+
+                print("    \x1b[1m\x1b[37mElapsed time:\x1b[0m\x1b[37m",
+                      str(time.time()-_global_start_time)[:6], 's\x1b[0m')
+
+            else:
+                sys.stdout.write(out + end)
+
+    except: # -------------------------------------------------------------
         print(*strings, end=end)
 
     if flush:
         sys.stdout.flush()
+
+def ask(*question, **kwarg):
+    """
+    Ask a question from command line. Return the answer as a string.
+    See function `printc()` for the description of the options.
+    """
+    kwarg.update({'end': ' '})
+    if 'invert' not in kwarg.keys():
+        kwarg.update({'invert': True})
+    if 'box' in kwarg.keys():
+        kwarg.update({'box': ''})
+    printc(*question, **kwarg)
+    resp = input()
+    return resp
