@@ -32,6 +32,7 @@ import vtk, sys
 __all__ = []
 
 
+####################################################################################
 def tips():
     from vedo import colors, __version__
     msg  = " ==========================================================\n"
@@ -71,16 +72,16 @@ def tips():
 
     msg = " vedo " + __version__ + " "
     colors.printc(msg, invert=1, dim=1, end="")
-    vtkVers = ''
     try:
-        vtkVers = vtk.VTK_VERSION_NUMBER
-    except AttributeError:
         vtkVers = vtk.vtkVersion().GetVTKVersion()
+        # vtkVers = vtk.VTK_VERSION_NUMBER
+    except AttributeError:
+        vtkVers = 'unkonwn'
     msg = "| vtk " + str(vtkVers)
     msg += " | python " + str(sys.version_info[0]) + "." + str(sys.version_info[1])
     colors.printc(msg, invert=0, dim=1)
 
-
+####################################################################################
 # example web page for X3D
 x3d_html = """
 <html>
@@ -119,6 +120,8 @@ google-chrome --enable-webgl --use-gl=desktop
 </font></body>
 </html>
 """
+####################################################################################
+
 
 ####################################################################################
 _defs = ""
@@ -1337,5 +1340,24 @@ _defs = ""
 #     :width: 350 px
 #     :target: flag_labels.py_
 #     :alt: flag_labels.py
+
+# .. |whiskers.py| replace:: whiskers.py
+# .. _whiskers.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/.whiskerspy
+# .. |whiskers| image:: https://user-images.githubusercontent.com/32848391/95772479-170cd000-0cbd-11eb-98c4-20c5ca342cb8.png
+#     :width: 350 px
+#     :target: whiskers.py_
+#     :alt: whiskers.py
+
 # """
 
+######################################################################## template
+# .. |.py| replace:: .py
+# .. _.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/.py
+# .. || image::
+#     :width: 350 px
+#     :target: .py_
+#     :alt: .py
+########################################################################
+
+
+
