@@ -2,6 +2,7 @@ from __future__ import division, print_function
 import vtk
 from vtk.util.numpy_support import vtk_to_numpy
 import os
+import glob
 import numpy as np
 
 import vedo.utils as utils
@@ -32,9 +33,9 @@ __all__ = [
     "loadStructuredGrid",
     "loadRectilinearGrid",
     "loadUnStructuredGrid",
-    "write",
+    "loadTransform",
     "writeTransform",
-    "readTransform",
+    "write",
     "exportWindow",
     "importWindow",
     "screenshot",
@@ -87,7 +88,6 @@ def load(inputobj, unpack=True, force=False):
     elif isinstance(inputobj, str) and inputobj.startswith('https://'):
         flist = [inputobj]
     else:
-        import glob
         flist = sorted(glob.glob(inputobj))
 
     for fod in flist:
@@ -141,13 +141,13 @@ def load(inputobj, unpack=True, force=False):
     if len(acts) == 1:
         if not acts[0]:
             colors.printc("\times Error in load(): cannot load", inputobj, c='r')
-        settings.collectable_actors.append(acts[0])
+        # settings.collectable_actors.append(acts[0])
         return acts[0]
     elif len(acts) == 0:
         colors.printc("\times Error in load(): cannot load", inputobj, c='r')
         return None
     else:
-        settings.collectable_actors += acts
+        # settings.collectable_actors += acts
         return acts
 
 
@@ -327,8 +327,9 @@ def download(url, force=False, verbose=True):
                                  os.path.basename(basename))
 
     if force==False and os.path.exists(tmp_file.name):
-        if verbose: colors.printc("using cached file:", tmp_file.name)
-        #colors.printc("     (use force=True to force a new download)")
+        if verbose:
+            colors.printc("using cached file:", tmp_file.name)
+            #colors.printc("     (use force=True to force a new download)")
         return tmp_file.name
 
     try:
@@ -346,7 +347,6 @@ def download(url, force=False, verbose=True):
         output.write(response.read())
 
     if verbose: colors.printc(' done.')
-
     return tmp_file.name
 
 
@@ -892,6 +892,9 @@ def loadNumpy(inobj):
         if "objects" in data[0].keys():  # loading a full scene!!
             return importWindow(data[0]) # <-------
 
+        if 'type' not in data[0].keys():
+            return data[0] # it's a very normal numpy data object, just return it!
+
     else:
         data = inobj
 
@@ -908,7 +911,7 @@ def loadNumpy(inobj):
             for i in [0, 1, 2, 3]:
                for j in [0, 1, 2, 3]:
                    vm.SetElement(i, j, d['transform'][i,j])
-            obj.setTransform(vm)
+            obj.applyTransform(vm)
         elif 'position' in keys:
             obj.pos(d['position'])
 
@@ -1112,18 +1115,22 @@ def write(objct, fileoutput, binary=True):
     elif isinstance(obj, (vtk.vtkPolyData, vtk.vtkImageData)):
         obj = objct
 
+    if hasattr(obj, 'filename'):
+        obj.filename = fileoutput
+
     fr = fileoutput.lower()
     if   fr.endswith(".vtk"):
         writer = vtk.vtkDataSetWriter()
     elif fr.endswith(".ply"):
         writer = vtk.vtkPLYWriter()
-        pscal = obj.GetPointData().GetScalars()
-        if not pscal:
-            pscal = obj.GetCellData().GetScalars()
-        if pscal and pscal.GetName():
-            writer.SetArrayName(pscal.GetName())
+        writer.AddComment("PLY file generated by vedo")
         lut = objct.GetMapper().GetLookupTable()
         if lut:
+            pscal = obj.GetPointData().GetScalars()
+            if not pscal:
+                pscal = obj.GetCellData().GetScalars()
+            if pscal and pscal.GetName():
+                writer.SetArrayName(pscal.GetName())
             writer.SetLookupTable(lut)
     elif fr.endswith(".stl"):
         writer = vtk.vtkSTLWriter()
@@ -1307,9 +1314,9 @@ def writeTransform(inobj, filename='transform.mat', comment=''):
         f.write('\n')
     return
 
-def readTransform(filename):
+def loadTransform(filename):
     """
-    Read a ``vtkTransform`` from a file.mat.
+    Load a ``vtkTransform`` from a file.mat.
 
     Returns
     -------
@@ -1573,7 +1580,38 @@ def screenshot(filename="screenshot.png", scale=None, returnNumpy=False):
     """
     if not settings.plotter_instance or not settings.plotter_instance.window:
         colors.printc('\bomb screenshot(): Rendering window is not present, skip.', c='r')
-        return
+        return settings.plotter_instance
+
+    if filename.endswith('.pdf'):
+        writer = vtk.vtkGL2PSExporter()
+        writer.SetRenderWindow(settings.plotter_instance.window)
+        writer.Write3DPropsAsRasterImageOff()
+        writer.SilentOn()
+        writer.SetSortToBSP()
+        writer.SetFileFormatToPDF()
+        writer.SetFilePrefix(filename.replace('.pdf',''))
+        writer.Write()
+        return settings.plotter_instance ##########
+    elif filename.endswith('.svg'):
+        writer = vtk.vtkGL2PSExporter()
+        writer.SetRenderWindow(settings.plotter_instance.window)
+        writer.Write3DPropsAsRasterImageOff()
+        writer.SilentOn()
+        writer.SetSortToBSP()
+        writer.SetFileFormatToSVG()
+        writer.SetFilePrefix(filename.replace('.svg',''))
+        writer.Write()
+        return settings.plotter_instance ##########
+    elif filename.endswith('.eps'):
+        writer = vtk.vtkGL2PSExporter()
+        writer.SetRenderWindow(settings.plotter_instance.window)
+        writer.Write3DPropsAsRasterImageOff()
+        writer.SilentOn()
+        writer.SetSortToBSP()
+        writer.SetFileFormatToEPS()
+        writer.SetFilePrefix(filename.replace('.eps',''))
+        writer.Write()
+        return settings.plotter_instance ##########
 
     if scale is None:
         scale = settings.screeshotScale
@@ -1604,15 +1642,19 @@ def screenshot(filename="screenshot.png", scale=None, returnNumpy=False):
     if filename.lower().endswith('.png'):
         writer = vtk.vtkPNGWriter()
         writer.SetFileName(filename)
-    elif filename.lower().endswith('.jpg'):
+        writer.SetInputData(w2if.GetOutput())
+        writer.Write()
+    elif filename.lower().endswith('.jpg') or filename.lower().endswith('.jpeg'):
         writer = vtk.vtkJPEGWriter()
         writer.SetFileName(filename)
-    else:
+        writer.SetInputData(w2if.GetOutput())
+        writer.Write()
+    else: #add .png
         writer = vtk.vtkPNGWriter()
         writer.SetFileName(filename+'.png')
-    writer.SetInputData(w2if.GetOutput())
-    writer.Write()
-    return writer.GetFileName()
+        writer.SetInputData(w2if.GetOutput())
+        writer.Write()
+    return settings.plotter_instance
 
 
 class Video:
@@ -1622,7 +1664,8 @@ class Video:
     :param str name: name of the output file.
     :param int fps: set the number of frames per second.
     :param float duration: set the total `duration` of the video and recalculates `fps` accordingly.
-    :param str ffmpeg: set path to ffmpeg program. Default value considers ffmpeg is in the path.
+    :param str ffmpeg: set path to ffmpeg program.
+        Default value assumes ffmpeg command is in the path.
 
     |makeVideo| |makeVideo.py|_
     """
@@ -1667,7 +1710,8 @@ class Video:
                azimuth_range=(0,359),
                zoom=None,
                cam1=None, cam2=None):
-        """Automatic shooting of the video with rotation and elevation ranges.
+        """
+        Automatic shooting of a static scene by specifying rotation and elevation ranges.
 
         :param list elevation_range: initial and final elevation angles
         :param list azimuth_range: initial and final azimuth angles
@@ -1713,6 +1757,7 @@ class Video:
             vp.camera.Azimuth(azimuth_range[0])
 
         vp.show(resetcam=False, interactive=False)
+        vp.renderer.ResetCamera()
 
         n = self.fps * self.duration
         for i in range(int(n)):
