@@ -3,7 +3,7 @@ import vtk
 import numpy as np
 from vtk.util.numpy_support import numpy_to_vtk, vtk_to_numpy
 
-import vedo.docs as docs
+import vedo
 import vedo.settings as settings
 import vedo.utils as utils
 import vedo.colors as colors
@@ -11,10 +11,8 @@ import vedo.shapes as shapes
 import vedo.addons as addons
 from vedo.assembly import Assembly
 from vedo.mesh import Mesh, merge
-from vedo.pointcloud import Points
-from vedo.base import BaseActor
 
-__doc__ = """Plotting utility functions.""" + docs._defs
+__doc__ = """Plotting utility functions.""" + vedo.docs._defs
 
 __all__ = [
     "plot",
@@ -486,7 +484,7 @@ def _plotxy(
     pad=0.05,
     axes={},
 ):
-    ncolls = len(settings.collectable_actors)
+    # ncolls = len(settings.collectable_actors)
 
     line=False
     if lw>0:
@@ -568,7 +566,7 @@ def _plotxy(
         l = shapes.KSpline(data).lw(lw).c(lc).alpha(la)
         acts.append(l)
     elif line:
-        l = shapes.Line(data, lw=lw, c=lc, alpha=la)
+        l = shapes.Line(data, c=lc, alpha=la).lw(lw)
         acts.append(l)
 
     if marker:
@@ -724,8 +722,8 @@ def _plotxy(
     asse.zmax = offs * 3  # z-order
     asse.name = "plotxy"
 
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
-    settings.collectable_actors.append(asse)
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors.append(asse)
     return asse
 
 
@@ -736,7 +734,7 @@ def _plotFxy(
     zlim=(None, None),
     showNan=True,
     zlevels=10,
-    c="b",
+    c=None,
     bc="aqua",
     alpha=1,
     texture="paper4",
@@ -751,9 +749,12 @@ def _plotFxy(
             exec(code, namespace)
             z = namespace["zfunc"]
         except:
-            colors.printc("Syntax Error in plotFxy()", c='r')
+            colors.printc("Syntax Error in _plotFxy()", c='r')
             return None
 
+    if c is not None:
+        texture = None # disable
+
     ps = vtk.vtkPlaneSource()
     ps.SetResolution(bins[0], bins[1])
     ps.SetNormal([0, 0, 1])
@@ -801,20 +802,20 @@ def _plotFxy(
         a = tmpact2.cutWithPlane((0, 0, zlim[1]), (0, 0, -1))
         poly = a.polydata()
 
-    if c is None:
-        elev = vtk.vtkElevationFilter()
-        elev.SetInputData(poly)
-        elev.Update()
-        poly = elev.GetOutput()
+    cmap=''
+    if c in colors._mapscales_cmaps:
+        cmap = c
+        c = None
+        bc= None
 
     mesh = Mesh(poly, c, alpha).computeNormals().lighting("plastic")
-    if c is None:
-        mesh.getPointArray("Elevation")
 
+    if cmap:
+        mesh.addElevationScalars().cmap(cmap)
     if bc:
         mesh.bc(bc)
-
-    mesh.texture(texture)
+    if texture:
+        mesh.texture(texture)
 
     acts = [mesh]
     if zlevels:
@@ -1099,7 +1100,7 @@ def _plotSpheric(rfunc, normalize=True, res=33, scalarbar=True, c="grey", alpha=
         xm = np.max([np.max(pts[0]), 1])
         ym = np.max([np.abs(np.max(pts[1])), 1])
         ssurf.mapper().SetScalarRange(np.min(newr), np.max(newr))
-        sb3d = ssurf.addScalarBar3D(sx=xm * 0.07, sy=ym, c='k')
+        sb3d = ssurf.addScalarBar3D(sx=xm * 0.07, sy=ym, c='k').scalarbar
         sb3d.rotateX(90).pos(xm * 1.1, 0, -0.5)
     else:
         sb3d = None
@@ -1140,7 +1141,7 @@ def _histogram1D(
     axes={},
     bc="k",
 ):
-    ncolls = len(settings.collectable_actors)
+    # ncolls = len(settings.collectable_actors)
 
     # purge NaN from data
     validIds = np.all(np.logical_not(np.isnan(data)))
@@ -1370,8 +1371,8 @@ def _histogram1D(
     asse.freqs = fs
     asse.name = "histogram1D"
 
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
-    settings.collectable_actors.append(asse)
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors.append(asse)
     return asse
 
 
@@ -1397,7 +1398,7 @@ def _histogram2D(
     axes=True,
     bc="k",
 ):
-    ncolls = len(settings.collectable_actors)
+    # ncolls = len(settings.collectable_actors)
     offs = 0  # z offset
 
     if format is not None:  # reset to allow meaningful overlap
@@ -1520,8 +1521,8 @@ def _histogram2D(
     asse.zmax = offs * 3  # z-order
     asse.name = "histogram2D"
 
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
-    settings.collectable_actors.append(asse)
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors.append(asse)
     return asse
 
 
@@ -1945,7 +1946,7 @@ def quiver(
 
     |quiver| |quiver.py|_
     """
-    if isinstance(points, Points):
+    if isinstance(points, vedo.Points):
         points = points.points()
     else:
         points = np.array(points)
@@ -2001,7 +2002,7 @@ def violin(
 
     |histo_violin| |histo_violin.py|_
     """
-    ncolls = len(settings.collectable_actors)
+    # ncolls = len(settings.collectable_actors)
 
     fs, edges = np.histogram(values, bins=bins, range=vlim)
     mine, maxe = np.min(edges), np.max(edges)
@@ -2062,8 +2063,65 @@ def violin(
     asse.base = np.array([0, 0, 0])
     asse.top = np.array([0, 1, 0])
     asse.name = "violin"
-    settings.collectable_actors = settings.collectable_actors[:ncolls]
-    settings.collectable_actors.append(asse)
+    # settings.collectable_actors = settings.collectable_actors[:ncolls]
+    # settings.collectable_actors.append(asse)
+    return asse
+
+
+def whisker(data,
+            s=0.25,
+            c='k',
+            lw=2,
+            bc='blue',
+            alpha=0.25,
+            r=3,
+            jitter=True,
+            horizontal=False,
+):
+    """
+    Generate a "whisker" bar from a 1-dimensional dataset.
+
+    :param float s: size of the box
+    :param c: color of the lines
+    :param float lw: line width
+    :param bc: color of the box
+    :param float alpha: transparency of the box
+    :param float r: point radius in pixels (use value 0 to disable)
+    :param bool jitter: add some randomness to points to avoid overlap
+    :param bool horizontal: set horizontal layout
+
+    |whiskers| |whiskers.py|_
+    """
+    xvals = np.zeros_like(data)
+    if jitter:
+        xjit = np.random.randn(len(xvals))*s/9
+        xjit = np.clip(xjit, -s/2.1, s/2.1)
+        xvals += xjit
+
+    dmean = np.mean(data)
+    dq05 = np.quantile(data, 0.05)
+    dq25 = np.quantile(data, 0.25)
+    dq75 = np.quantile(data, 0.75)
+    dq95 = np.quantile(data, 0.95)
+
+    pts = None
+    if r: pts = shapes.Points([xvals, data], c=c, r=r)
+
+    rec = shapes.Rectangle([-s/2, dq25],[s/2, dq75], c=bc, alpha=alpha)
+    rl = shapes.Line([[-s/2, dq25],[s/2, dq25],[s/2, dq75],[-s/2, dq75]], closed=True)
+    l1 = shapes.Line([0,dq05,0], [0,dq25,0], c=c, lw=lw)
+    l2 = shapes.Line([0,dq75,0], [0,dq95,0], c=c, lw=lw)
+    lm = shapes.Line([-s/2, dmean], [s/2, dmean])
+    lns = merge(l1,l2,lm,rl)
+    asse = Assembly([lns, rec, pts])
+    if horizontal:
+        asse.rotateZ(-90)
+    asse.name = "Whisker"
+    asse.info['mean'] = dmean
+    asse.info['quantile_05'] = dq05
+    asse.info['quantile_25'] = dq25
+    asse.info['quantile_75'] = dq75
+    asse.info['quantile_95'] = dq95
     return asse
 
 
@@ -2085,8 +2143,8 @@ def streamplot(X, Y, U, V, direction="both",
 
     |plot7_stream| |plot7_stream.py|_
     """
-    from vedo.volume import Volume
-    from vedo.base import streamLines
+    # from vedo.volume import Volume
+    # from vedo.base import streamLines
 
     n = len(X)
     m = len(Y[0])
@@ -2099,7 +2157,7 @@ def streamplot(X, Y, U, V, direction="both",
 
     field = np.sqrt(U * U + V * V)
 
-    vol = Volume(field, dims=(n, n, 1))
+    vol = vedo.Volume(field, dims=(n, n, 1))
 
     uf = np.ravel(U, order="F")
     vf = np.ravel(V, order="F")
@@ -2109,7 +2167,7 @@ def streamplot(X, Y, U, V, direction="both",
     if len(probes) == 0:
         probe = shapes.Grid(pos=((n-1)/2,(n-1)/2,0), sx=n-1, sy=n-1, resx=n-1, resy=n-1)
     else:
-        if isinstance(probes, Points):
+        if isinstance(probes, vedo.Points):
             probes = probes.points()
         else:
             probes = np.array(probes)
@@ -2118,15 +2176,14 @@ def streamplot(X, Y, U, V, direction="both",
         sv = [(n - 1) / (xmax - xmin), (n - 1) / (ymax - ymin), 1]
         probes = probes - [xmin, ymin, 0]
         probes = np.multiply(probes, sv)
-        probe = shapes.Points(probes)
+        probe = vedo.Points(probes)
 
-    stream = streamLines(
-                            vol.imagedata(),
-                            probe,
-                            tubes={"radius": lw, "varyRadius": mode,},
-                            lw=lw,
-                            maxPropagation=maxPropagation,
-                            direction=direction,
+    stream = vedo.base.streamLines( vol.imagedata(),
+                                    probe,
+                                    tubes={"radius": lw, "varyRadius": mode,},
+                                    lw=lw,
+                                    maxPropagation=maxPropagation,
+                                    direction=direction,
     )
     if c is not None:
         stream.color(c)
@@ -2139,7 +2196,7 @@ def streamplot(X, Y, U, V, direction="both",
     return stream
 
 
-def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True):
+def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True, dots=True):
     """
     Return a ``vtkXYPlotActor`` that is a plot of `x` versus `y`,
     where `points` is a list of `(x,y)` points.
@@ -2152,7 +2209,6 @@ def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True):
         - 4, bottomright.
     """
     if len(points) == 2:  # passing [allx, ally]
-        # points = list(zip(points[0], points[1]))
         points = np.stack((points[0], points[1]), axis=1)
 
     c = colors.getColor(c)  # allow different codings
@@ -2186,9 +2242,20 @@ def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True):
     plot.SetXTitle(title)
     plot.SetYTitle("")
     plot.ExchangeAxesOff()
-    plot.PlotPointsOn()
+    plot.SetPlotPoints(dots)
+
     if not lines:
         plot.PlotLinesOff()
+
+    if isinstance(pos, str):
+        spos = 2
+        if "top" in pos:
+            if "left" in pos: spos=1
+            elif "right" in pos: spos=2
+        elif "bottom" in pos:
+            if "left" in pos: spos=3
+            elif "right" in pos: spos=4
+        pos = spos
     if pos == 1:
         plot.GetPositionCoordinate().SetValue(0.0, 0.8, 0)
     elif pos == 2:
@@ -2199,6 +2266,7 @@ def cornerPlot(points, pos=1, s=0.2, title="", c="b", bg="k", lines=True):
         plot.GetPositionCoordinate().SetValue(0.76, 0.0, 0)
     else:
         plot.GetPositionCoordinate().SetValue(pos[0], pos[1], 0)
+
     plot.GetPosition2Coordinate().SetValue(s, s, 0)
     return plot
 
@@ -2216,6 +2284,7 @@ def cornerHistogram(
     pos=1,
     s=0.2,
     lines=True,
+    dots=False,
 ):
     """
     Build a histogram from a list of values in n bins.
@@ -2230,6 +2299,9 @@ def cornerHistogram(
         - 4, bottomright,
         - (x, y), as fraction of the rendering window
     """
+    if hasattr(values, '_data'):
+        values = vtk_to_numpy(values._data.GetPointData().GetScalars())
+
     fs, edges = np.histogram(values, bins=bins, range=vrange)
     if minbin:
         fs = fs[minbin:-1]
@@ -2239,11 +2311,13 @@ def cornerHistogram(
     for i in range(len(fs)):
         pts.append([(edges[i] + edges[i + 1]) / 2, fs[i]])
 
-    plot = cornerPlot(pts, pos, s, title, c, bg, lines)
+    plot = cornerPlot(pts, pos, s, title, c, bg, lines, dots)
     plot.SetNumberOfYLabels(2)
     plot.SetNumberOfXLabels(3)
     tprop = vtk.vtkTextProperty()
     tprop.SetColor(colors.getColor(bg))
+    tprop.SetFontFamily(vtk.VTK_FONT_FILE)
+    tprop.SetFontFile(settings.fonts_path + settings.defaultFont + '.ttf')
     tprop.SetOpacity(alpha)
     plot.SetAxisTitleTextProperty(tprop)
     plot.GetProperty().SetOpacity(alpha)
@@ -2308,7 +2382,7 @@ class DirectedGraph(Assembly):
         |graph_network| |graph_network.py|_
     """
     def __init__(self, **kargs):
-        BaseActor.__init__(self)
+        vedo.base.BaseActor.__init__(self)
 
         self.nodes = []
         self.edges = []
@@ -2326,7 +2400,9 @@ class DirectedGraph(Assembly):
 
         self.arrowScale = 0.15
         self.nodeLabelScale = None
-        self.edgeLabelScale   = None
+        self.nodeLabelJustify = "bottom-left"
+
+        self.edgeLabelScale = None
 
         self.mdg = vtk.vtkMutableDirectedGraph()
 
@@ -2478,7 +2554,7 @@ class DirectedGraph(Assembly):
         graphToPolyData.Update()
 
         dgraph = Mesh(graphToPolyData.GetOutput(0))
-        dgraph.clean() # important to clean and have the same nr of points as vtxs
+        # dgraph.clean() # WRONG!!! dont uncomment
         dgraph.flat().color(self._c).lw(2)
         dgraph.name = "DirectedGraph"
 
@@ -2523,6 +2599,7 @@ class DirectedGraph(Assembly):
                                     scale=self.nodeLabelScale,
                                     precision=0,
                                     font=self.font,
+                                    justify=self.nodeLabelJustify,
                                     )
         nodeLabels.color(self._c).pickable(True)
         nodeLabels.name = "DirectedGraphNodeLabels"
@@ -2536,10 +2613,9 @@ class DirectedGraph(Assembly):
         edgeLabels.color(self._c).pickable(True)
         edgeLabels.name = "DirectedGraphEdgeLabels"
 
-        # # if edgeLabels.N() == 0:
-        # edgeLabels = None
-        # exit()
-
-        Assembly.__init__(self, [dgraph, nodeLabels, edgeLabels, arrows])
+        Assembly.__init__(self, [dgraph,
+                                 nodeLabels,
+                                 edgeLabels,
+                                 arrows])
         self.name = "DirectedGraphAssembly"
         return self
