@@ -12,7 +12,7 @@ General settings.
     # Set a default for the font to be used for axes, comments etc.
     defaultFont = 'Normografo'
     # Options are:
-    #    Biysk, Bongas, Calco, Comae, Kanopus, Glasgo, Inversionz, LionelOfParis,
+    #    Biysk, Bongas, Calco, Comae, Kanopus, Glasgo, LionelOfParis,
     #    LogoType, Normografo, Quikhand, SmartCouric, Theemim, VictorMono, VTK.
     fontIsMono = True  # mono means that all letters occupy the same space slot horizontally
     fontHSpacing = 1   # an horizontal stretching factor (affects both letters and words)
@@ -21,9 +21,7 @@ General settings.
     # Scale magnification of the screenshot (must be an integer)
     screeshotScale = 1
     screenshotTransparentBackground = False
-
-    # Sometimes setting this to True gives better results
-    screeshotLargeImage = False
+    screeshotLargeImage = False # Sometimes setting this to True gives better results
 
     # Recompute vertex and cell normals
     computeNormals = None
@@ -82,6 +80,7 @@ General settings.
 
     # Turn on/off the automatic repositioning of lights as the camera moves.
     lightFollowsCamera = False
+    twoSidedLighting = True
 
     # Turn on/off rendering of translucent material with depth peeling technique.
     useDepthPeeling = False
@@ -192,6 +191,7 @@ visibleGridEdges = False
 
 # Turn on/off the automatic repositioning of lights as the camera moves.
 lightFollowsCamera = False
+twoSidedLighting = True
 
 # Turn on/off rendering of translucent material with depth peeling technique.
 #https://lorensen.github.io/VTKExamples/site/Cxx/Visualization/CorrectlyRenderTranslucentGeometry
@@ -259,12 +259,12 @@ fonts_path = os.path.join(installdir, "fonts/")
 fonts = []
 
 #datadir = "/home/musy/Dropbox/Public/vktwork/vedo_data/"
+#print('\ndatadir=',datadir)
 datadir = "https://vedo.embl.es/examples/data/"
 
 plotter_instances = []
-
 plotter_instance = None
-collectable_actors = []
+collectable_actors = [] # OBSOLETE
 
 ####################################################################################
 def embedWindow(backend='k3d', verbose=True):
@@ -338,7 +338,7 @@ def _init():
 
     plotter_instance = None
     plotter_instances = []
-    collectable_actors = []
+    collectable_actors = [] # OBSOLETE
 
     for f in os.listdir(textures_path):
         tfn = f.split(".")[0]
