@@ -2,20 +2,19 @@ from __future__ import division, print_function
 import glob, os
 import numpy as np
 import vtk
+from vtk.util.numpy_support import numpy_to_vtk, vtk_to_numpy
 import vedo.colors as colors
 import vedo.docs as docs
 import vedo.utils as utils
-from vtk.util.numpy_support import numpy_to_vtk
 from vedo.mesh import Mesh
 from vedo.base import BaseGrid
-__doc__ = (
-    """
-Submodule extending the ``vtkVolume`` object functionality.
-"""
+
+__doc__ = ("""Submodule extending the ``vtkVolume`` object functionality."""
     + docs._defs
 )
 
-__all__ = ["Volume",
+__all__ = [
+           "Volume",
            "mesh2Volume",
            "volumeFromMesh",
            "interpolateToVolume",
@@ -23,7 +22,6 @@ __all__ = ["Volume",
           ]
 
 
-
 def mesh2Volume(mesh, spacing=(1, 1, 1)):
     """
     Convert a mesh it into a ``Volume``
@@ -125,13 +123,14 @@ def volumeFromMesh(mesh, bounds=None, dims=(20,20,20), signed=True, negate=False
     vol.name = "VolumeFromMesh"
     return vol
 
-def interpolateToVolume(mesh, kernel='shepard', radius=None,
-                       bounds=None, nullValue=None,
-                       dims=(20,20,20)):
+def interpolateToVolume(mesh, kernel='shepard',
+                        radius=None, N=None,
+                        bounds=None, nullValue=None,
+                        dims=(25,25,25)):
     """
     Generate a ``Volume`` by interpolating a scalar
     or vector field which is only known on a scattered set of points or mesh.
-    Available interpolation kernels are: shepard, gaussian, voronoi, linear.
+    Available interpolation kernels are: shepard, gaussian, or linear.
 
     :param str kernel: interpolation kernel type [shepard]
     :param float radius: radius of the local search
@@ -146,6 +145,10 @@ def interpolateToVolume(mesh, kernel='shepard', radius=None,
     else:
         output = mesh.polydata()
 
+    if radius is None and not N:
+        colors.printc("Error in interpolateToVolume(): please set either radius or N", c='r')
+        raise RuntimeError
+
     # Create a probe volume
     probe = vtk.vtkImageData()
     probe.SetDimensions(dims)
@@ -156,8 +159,8 @@ def interpolateToVolume(mesh, kernel='shepard', radius=None,
                      (bounds[3]-bounds[2])/dims[1],
                      (bounds[5]-bounds[4])/dims[2])
 
-    if radius is None:
-        radius = min(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])/3
+    # if radius is None:
+    #     radius = min(bounds[1]-bounds[0], bounds[3]-bounds[2], bounds[5]-bounds[4])/3
 
     locator = vtk.vtkPointLocator()
     locator.SetDataSet(output)
@@ -166,25 +169,30 @@ def interpolateToVolume(mesh, kernel='shepard', radius=None,
     if kernel == 'shepard':
         kern = vtk.vtkShepardKernel()
         kern.SetPowerParameter(2)
-        kern.SetRadius(radius)
     elif kernel == 'gaussian':
         kern = vtk.vtkGaussianKernel()
-        kern.SetRadius(radius)
-    elif kernel == 'voronoi':
-        kern = vtk.vtkVoronoiKernel()
     elif kernel == 'linear':
         kern = vtk.vtkLinearKernel()
-        kern.SetRadius(radius)
     else:
         print('Error in interpolateToVolume, available kernels are:')
-        print(' [shepard, gaussian, voronoi, linear]')
+        print(' [shepard, gaussian, linear]')
         raise RuntimeError()
 
+    if radius:
+        kern.SetRadius(radius)
+
     interpolator = vtk.vtkPointInterpolator()
     interpolator.SetInputData(probe)
     interpolator.SetSourceData(output)
     interpolator.SetKernel(kern)
     interpolator.SetLocator(locator)
+
+    if N:
+        kern.SetNumberOfPoints(N)
+        kern.SetKernelFootprintToNClosest()
+    else:
+        kern.SetRadius(radius)
+
     if nullValue is not None:
         interpolator.SetNullValue(nullValue)
     else:
@@ -462,7 +470,6 @@ class Volume(vtk.vtkVolume, BaseGrid):
         ``volume.imagedata().GetPointData().GetScalars().Modified()``
         when all your modifications are completed.
         """
-        from vtk.util.numpy_support import vtk_to_numpy
         narray_shape = tuple(reversed(self._data.GetDimensions()))
         narray = vtk_to_numpy(self._data.GetPointData().GetScalars()).reshape(narray_shape)
         narray = np.transpose(narray, axes=[2, 1, 0])
@@ -1063,6 +1070,24 @@ class Volume(vtk.vtkVolume, BaseGrid):
         return self._update(imgm.GetOutput())
 
 
+    # def histogram(self, bins=25, component=0, logscale=True, normalized=False):
+    #     # seems not to work properly>???
+    #     hi = vtk.vtkImageHistogram()
+    #     hi.SetInputData(self._data)
+    #     hi.GenerateHistogramImageOff()
+    #     hi.SetNumberOfBins(bins)
+    #     hi.SetActiveComponent(component)
+    #     hi.Update()
+    #     arr = vtk_to_numpy(hi.GetHistogram())
+    #     tot = hi.GetTotal()
+    #     if logscale:
+    #         arr = np.log10(arr+1)
+    #         tot = np.log10(tot+bins)
+    #     if normalized:
+    #         arr = arr /tot
+    #     return arr
+
+
     def toPoints(self):
         """Extract all image voxels as points.
         This function takes an input ``Volume`` and creates an ``Mesh``
